import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib

# Define UUIDs for your service and characteristic
SERVICE_UUID = '00001801-0000-1000-8000-00805f9b34fb'
PAIRING_UUID = '00002a25-0000-1000-8000-00805f9b34fb'

class PairingCharacteristic(dbus.service.Object):
    def __init__(self, service, index):
        self.path = f"{service.path}/char{index}"
        self.service = service
        self.props = {
            'UUID': dbus.String(PAIRING_UUID),
            'Service': dbus.ObjectPath(service.path),
            'Flags': dbus.Array(['read', 'write'], signature='s')
        }
        super().__init__(service.bus, self.path)

    @dbus.service.method('org.bluez.GattCharacteristic1', in_signature='', out_signature='a{sv}')
    def GetProperties(self):
        return self.props

    @dbus.service.method('org.bluez.GattCharacteristic1', in_signature='ay', out_signature='ay')
    def ReadValue(self, options):
        return dbus.Array([dbus.Byte(0x00)], signature='y')

    @dbus.service.method('org.bluez.GattCharacteristic1', in_signature='aya{sv}', out_signature='')
    def WriteValue(self, value, options):
        print(f"Pairing request received: {value}")
        # Handle pairing logic here

class BLEService(dbus.service.Object):
    def __init__(self, bus, index):
        self.path = f"/org/bluez/example/service{index}"
        self.props = {
            'UUID': dbus.String(SERVICE_UUID),
            'Primary': dbus.Boolean(True),
        }
        super().__init__(bus, self.path)
        self.characteristics = [PairingCharacteristic(self, 0)]

    @dbus.service.method('org.bluez.GattService1', in_signature='', out_signature='a{sv}')
    def GetProperties(self):
        return self.props

    @dbus.service.method('org.bluez.GattService1', in_signature='', out_signature='')
    def Release(self):
        print(f"{self.path}: Released")

def main():
    DBusGMainLoop(set_as_default=True)
    bus = dbus.SystemBus()

    service = BLEService(bus, 0)

    print("BLE Service running. Press Ctrl+C to stop.")
    mainloop = GLib.MainLoop()
    mainloop.run()

if __name__ == '__main__':
    main()